/**
 * Avago ADNS5050 optical sensor driver
 * (c) DLC TTT Enterprises
 * Do what you want with this, but at least mention my name.
 */

#include <avr/io.h>
#include "device.h"
#include <util/delay.h>
#include "adns5050.h"

void InitADNS(void)
/*
 * Sets up the controller and does a reset on the ADNS-5050
 */
{
	DDRC |= 0x0C;
	DDRD |= 0x03;
	nADNSRS = 0;
	_delay_us(2);
	nADNSRS = 1;
}

char ADNS_wr(unsigned char addr, unsigned char data)
/*
 * ADNS5050 write
 * addr = register address
 * data = data to write there or nothing
 * returns data read back (or nothing if a write)
 * msb of address = 1 for a write, 0 for a read
 * 
 * We are operating at about 500K bps clock rate, it takes about 20us to 
 * do a full read/write session.  This can obviously be improved upon.
 * The ADNS-5050 reads and writes the msb first, so we work from the "left"
 * side to the "right" side when writing and the reverse when reading back.
 */
{
	char temp;
	int n;
	
	nADNSCS = 0;				// select the chip
	
	temp = addr;
	SCK = 0;					// start clock low
	DATA_OUT;					// set data line for output
	for (n=0; n<8; n++) {
		SCK = 0;
		_delay_us(1);
		if (temp & 0x80)
			SDOUT = 1;
		else
			SDOUT = 0;
		temp = (temp << 1);
		SCK = 1;
		_delay_us(1);			// short clock pulse
	}
	if (addr & 0x80) {			// This is a write, keep writing
		temp = data;
		for (n=0; n<8; n++) {
			SCK = 0;
			_delay_us(1);
			if (temp & 0x80)
				SDOUT = 1;
			else
				SDOUT = 0;
			temp = (temp << 1);
			SCK = 1;
			_delay_us(1);			// short clock pulse
		}
	}
	else {							// This is a read, switch to input
		DATA_IN;
		for (n=0; n<8; n++) {		// read back the data
			SCK = 0;
			_delay_us(1);
			SCK = 1;
			if (SDIN) {				// got a '1'
				temp|=1;
			}
			temp = (temp <<1);		// shift left
			_delay_us(1);
		}
	}
	nADNSCS = 1;				// de-select the chip
	return(temp);
}
