#ifndef ADNS5050_H_
#define ADNS5050_H_

#include "device.h"

//Hardware IO defs
typedef union port_c
{
	unsigned char value;						//whole 8 bits
	struct
	{
		unsigned char b0:1;						//Generic bits
		unsigned char b1:1;
		unsigned char b2:1;
		unsigned char b3:1;
		unsigned char b4:1;
		unsigned char b5:1;
		unsigned char b6:1;
		unsigned char b7:1;		
	};
} port_c;
#define PORTCbits (*((volatile port_c *)(&PORTC)))
#define PINCbits (*((volatile port_c *)(&PINC)))

typedef union port_d
{
	unsigned char value;						//whole 8 bits
	struct 
	{
		unsigned char b0:1;						//Generic bits
		unsigned char b1:1;
		unsigned char b2:1;
		unsigned char b3:1;
		unsigned char b4:1;
		unsigned char b5:1;
		unsigned char b6:1;
		unsigned char b7:1;		
	};
} port_d;
#define PORTDbits (*((volatile port_d *)(&PORTD)))
#define PINDbits (*((volatile port_d *)(&PIND)))

/*
 * pin assignments
 */
#define SCK PORTCbits.b3
#define SDIN PINCbits.b2
#define SDOUT PORTCbits.b2
#define nADNSCS PORTDbits.b0
#define nADNSRS PORTDbits.b1

// flippy for SDIO line
#define DATA_IN DDRC &= 0xFB
#define DATA_OUT DDRC |= 0x04
	
/*
 * Functions
 */
void InitADNS(void);
char ADNS_wr(unsigned char addr, unsigned char data);

/*
 * register addresses
 */
#define PROD_ID		0x00
#define REV_ID		0x01
#define MOTION		0x02
#define DELTA_X		0x03
#define DELTA_Y		0x04
#define SQUAL		0x05
#define P_GRABR		0x0B
#define P_GRABW		0x8B
#define M_CTRLW		0x8D
#define M_CTRLR		0x0D
#define M_CTRL2W	0x99
#define M_CTRL2R	0x19
#define LED_DC		0xA2
#define BURST		0x63
#define INV_ID		0x3F

/*
 * Data values
 */
#define ID5050		0x12		// product ID for ADNS5050
#define GOTMOT		0x80		// msb set, motion since last read
#define C_PDN		0x02		// (or) to power down
#define C_RES1K		0x01		// (or) for 1000 cpi 
#define DC_MODE		0x80		// LED always on

#endif /*ADNS5050_H_*/
